function [x,ifallo]=ansr02(R,b,piv)

%%%%%%%%%%%%
%Autor:Carlos Crespo bajo la direccin de Cecilia Pola
%Fecha:2/11/2020
%Referencia: funcion anrs02 FORTRAN de Eduardo Casas y Cecilia Pola.
%
% OBJETIVO: Este programa resuelve un sistema del tipo:
%                                 P'R'RPx=b 
% donde  P  es una matriz de permutacion y  R  es una matriz triangular 
% superior.
%
% Lista de llamada: R es una matriz triangular superior
%                   b es un vector N-dimensional con los terminos 
%                   independientes del sistema
%                   piv es un vector N-dimensional que almacena la
%                   informacin de la pivotacion, cambios de fila al
%                   multiplicar por la derecha (piv(I)=J indica que en la
%                   posicin i-esima va la fila j-esima).
%
% Lista de Salida: x es un vector N-dimensional que almacena la solucion
%                   del sistema.
%                   ifallo es un indicador  que vale:
%                       0 si se calcula la direccion
%                       1 si se produce un error
%%%%%%%%%%%%

x=zeros(size(R,1),1);
[u,ifail]=sollowtri(R',b(piv));
if ifail==0
    [w,ifail]=solupptri(R,u);
    if ifail==0
        x(piv)=w;
        ifallo=0;
    else
        ifallo=1;
        fprintf('Fallo calculando la direccion');
        return
    end
else
    ifallo=1;
    fprintf('Fallo calculando la direccion');
    return
end

