function [LT,ipvt,ind]=cholpar(H,n,Z,NColZ,modo)
%%%%%%%%%%%%
%Author:Carlos Crespo under the direction of Cecilia Pola
%Date:27/11/2020
%Reference: anfm03 FORTRAN function of Eduardo Casas and Cecilia Pola.
%
% OBJECTIVE: This program calculate the Cholesky factorization (LT'LT) with
% rows and columns pivotation for a matrix such as:
%                                PZ'HZP'
% where P is a matrix that turn over the columns.
%
%
% Input list:      H nxn-dimensional matrix
%                  n matrix size H
%                  Z nxNColZ-dimensional matrix. The columns are in reverse
%                  order
%                  NColZ number of columns of Z.
%                  modo is an indicator that takes the values:
%                   0: if the matrix is non-definite the factorization
%                      is not carried out.
%        other number:  the factorization is not out (partial or complete).
%
% Output list :     LT from the row ind+1 it contains the upper triangular
%                    matrix of the Cholesky factorization (the
%                    factorization could be partial in the non-definite
%                    case
%                   ipvt indicate the exchange of the columns and rows
%                   ind variable that takes the next values:
%                       n: if the factorizatiaon has not been done and
%                       modo=0
%                       (-1,ND+1): if the factorization is finished. ND-ind
%                       is the size of the non-zero part of LT.
%                       (-ND-1,0): the factorization is not finished. A
%                       negative element in the position -IND in the
%                       diagonal of the matrix LT has been found.
%                       (-2*ND-1,-ND): the factorization is not complete
%                       because the matrix is indefinite: LT has in the
%                       position -IND-ND in the diagonal a null element and
%                       a non-zero element in the position (-ND-IND,
%                       -ND-IND+1)
%%%%%%%%%%%%
eps1=eps^0.75;
ind=0;
LT=zeros(n,n);
Z=Z(:,NColZ:-1:1);
%[M,p]=chol(Z'*H*Z);
if NColZ==0
    LT=[];
    ipvt=[];
    ind=0;
    return
end
if ind==0
    ndim=NColZ;
    iifact=0;
    ipvt=(1:ndim);
end
% if p==0
%     LT=M;
%     fprintf('positive definite matrix')
%     ind=0;
%     return;
% end
smax=1;
for i=1:ndim
    if ind==0
        ii=i;
    end
    s=Z(1:n,i)'*H(1:n,1:n)*Z(1:n,i);
    if modo==0 && s<-eps1
        ind=n;
        return
    end
    LT(ii,i)=s;
    smax=max(abs(s),smax);
end
if ndim==1
    ik=ind+1;
    s=LT(ik,1);
    if s>eps1
        LT(ik,1)=sqrt(s);
        ind=0;
    elseif s<-eps1
        ind=-1;
    else
        ind=1;
    end
    return
end
eps0=eps*smax;
eps1=eps0*ind;
beta=max(eps0*ndim*10,sqrt(smax)*1.2);
iibeta=0;
s1=0;
for k=1:ndim-1
    LT
    iter=k;
    eps1=eps1+eps0;
    kk=k+1;
    ik=k;
    ik0=ik-1;
    if iifact==0
        LT(k,k)=LT(k,k)-LT(1:ik0,k)'*LT(1:ik0,k);
        sk=LT(k,k);
        if s1>beta || sk<=eps1
            iifact=1;
            for i=kk:ndim
                LT(i,i)=LT(i,i)-LT(1:ik0,i)'*LT(1:ik0,i);
            end
        end
    else
        sk=LT(ik,k);
    end
    if s1>beta
        s=-1;
        iibeta=1;
    elseif iifact==1
        j=k;
        s=sk;
        for i=kk:ndim
            ii=i+ind;
            Rii=LT(ii,i);
            if Rii>s
                j=i;
                s=Rii;
            end
        end
    else
        s=sk;
        j=k;
    end
    if s>eps1
        if iifact==1
            index=LT(1:ik0,k);
            LT(1:ik0,k)=LT(1:ik0,j);
            LT(1:ik0,j)=index;
            auxipvt=ipvt(k);
            ipvt(k)=ipvt(j);
            ipvt(j)=auxipvt;
            LT(ind+j,j)=sk;
            l=ipvt(k);
        else
            l=k;
        end
        sk=sqrt(s);
        LT(ik,k)=sk;
        w=H*Z(1:n,l);
        s1=0;
        for i=kk:ndim
            j=ipvt(i);
            s=Z(1:n,j)'*w;
            if ik0>0
                s=s-LT(1:ik0,i)'*LT(1:ik0,k);
            end
            Rik=s/sk;
            s1=max(s1,abs(Rik));
            LT(ik,i)=Rik;
            ii=i;
            if iifact==1
                LT(ii,i)=LT(ii,i)-Rik*Rik;
            end
        end
    else
        s=sk;
        j=k;
        for i=kk:ndim
            ii=ind+i;
            Rii=LT(ii,i);
            if Rii<s
                j=i;
                s=Rii;
            end
        end
        if s<-eps1
            if modo==0
                ind=n;
                return;
            end
            index=LT(1:ik0,k);
            LT(1:ik0,k)=LT(1:ik0,j);
            LT(1:ik0,j)=index;
            auxipvt=ipvt(j);
            ipvt(j)=ipvt(k);
            ipvt(k)=auxipvt;
            LT(ind+j,j)=sk;
            LT(ik,k)=s;
            ind=-k;
            return;
            
        else
            for j=k:ndim-1
                nj=ipvt(i);
                ij=j+ind;
                w=H*Z(1:n,nj);
                for i=j+1:ndim
                    j=ipvt(i);
                    s1=Z(:,ipvt(i))'*w;
                    if ik0>0
                        s1=s1-LT(1:ik0,i)'*LT(1:ik0,j);
                    end
                    LT(ij,i)=s1; 
                    s1=abs(s1);
                    if s1>s
                        s=s1;
                        l=i;
                    end
                end
                if s>eps1
                    if modo==0
                        ind=n;
                        return
                    end
                    index=LT(1:ik0,k);
                    index2=LT(1:ik0,kk);
                    LT(1:ik0,k)=LT(1:ik0,j);
                    LT(1:ik0,j)=index;
                    LT(1:ik0,kk)=LT(1:ik0,l);
                    LT(1:ik0,l)=index2;
                    LT(ik,kk)=LT(ij,l);
                    auxipvt=ipvt(j);
                    ipvt(j)=ipvt(k);
                    ipvt(k)=auxipvt;
                    auxipvt=ipvt(l);
                    ipvt(l)=ipvt(kk);
                    ipvt(kk)=auxipvt;
                    ind=-ndim-k;
                    return
                end
            end
            ind=ndim-k+1;
            return
        end
    end
end
eps1=eps0+eps1;
in=ndim+ind;
if iifact==0
    LT(in,in)=LT(in,in)-LT(1:in-1,in)'*LT(1:in-1,in);
end
s=LT(in,ndim);
if s>eps1
    LT(in,ndim)=sqrt(s);
    ind=0;
elseif s<-eps1
    ind=-ndim;
else
    ind=1;
end
end


