function [x,fx,ind,lambda,ire,ipvt,m,q,r,iter]=quaprog(H,p,C,b,lb,ub,x,n,neq,nin,ira,modo,imp,iter)
%%%%%%%%%%%%
%Author:Carlos Crespo under the direction of Cecilia Pola
%Date:27/5/2021
%Reference: optr03 FORTRAN function of Eduardo Casas and Cecilia Pola.
%OBJECTIVE: This program minimizes the quadratic objective function
%subject to linear constraints (equality, inequality and bound constraints)
%with the following structure: 
%                        F(x)=0.5*x'*H*x+p'*x
%         
% Input list:
% - H: nxn-dimensional matrix. The Hessian matrix
% - p: n vector which contains the coefficients of the linear term in F.
% - C: nx(neq+nin) matrix which contains the coefficients of the equality
%     and general inequality constraints
% - b: neq+nin vector which contains the coefficients of the independent
%      terms of the equality and general inequality constraints
% - lb: n vector that, if ira=1 or 3 contains the lower bounds for x
% - ub: n vector that, if ira=2 or 3 contains the upper bounds for x
% - x: n-vector which contains the inital feasible vector
% - n: number of variables of the problem
% - neq: number of equality constraints 
% - nin: number of general inequality constraints 
% - ira: This variable indicates if there are bound constraints:
%       *0: there are no bound constraints 
%       *1: there are only lower bound constraints 
%       *2: there are only upper bound constraints 
%       *3: there are both type of bound constraints 
% -modo: this value have to be 2 and indicates that an initial feasible
% point is given
% - imp: If imp=1 the details of the process is shown. In other case only the
% final results are shown 
% -iter: maximum number of iterations
%
% Output list :    
% - x: the best estimate of the solution which minimize the objective function
% - fx: value of the objective evaluated at x.
% - ind: indicates the reason why the process ends with the values:
%    *1 the limited number of iterations has been reached
%    *0 a minimum has been foud
%    *<> implies numerical difficulties
% -lambda: vector containing the Lagrange multipliers for the obtained
%     point
% - ire: Indicates if the constraints are in the working set 
%      *0 it is not in the working set
% -ipvt: indicates the indexes of the constraints contained in the working set
% -m: Number of constraints in the working set
% -q,r: QR factorization of the working set matrix
% -iter: number of iterations used by the process
%%%%%%%%%%%%
eps1=eps^.75;
eps0=eps^.9;
gigant=realmax;
gig1=sqrt(gigant);
lambda=[];
if ira>0
    for i=1:n
        if ira==3
            if(lb>=-gig1 & ub(i)<=gig1 & lb(i)>ub(i))
                fprintf('Error en las cotas, lb(i)>ub(i) para i=%e \n',i)
                ind=-4;
                return
            end
        end
    end
end
iver=-1;
n1=n+1;
n10=0;
w=zeros(2*n,1);
if modo~=3 || modo~=0
    mi1=neq;
end
mid=neq+nin;
nmd=n+nin;
nd1=nmd+1;
nd=nd1;%
icd=nd+n;
icol=0;
id=0;
ind=0;
icicla=0;
icol1=0;
icol2=0;
iicol=0;
info=0;
if iter<=0
    itermax=14*(n+mid);
else
    itermax=iter;
end
iter=1;
icont=0;
if ira==0
    ire(1:n)=0;
end
ipvt=zeros(n+neq+nin+1,1);
if mid==0 && (modo==2 || (modo==1 && ira==0))
    m=0;
    q=eye(n);
    r=[];
elseif mid~=0 && (modo==2 || (modo==1 && ira==0))%%
    if neq>0
        [q,r]=qr(C(1:n,1:neq));
        for i=1:neq
            ipvt(i)=i;
            if imp==1
                fprintf('\n constraint %3d is added ',i+n);
            end
        end
        m=neq;
    else
        m=0;
        q=eye(n);
        r=[];
    end
end
if modo==2
    if ira>=1
        for i=1:n
            ire(i)=0;
            if ira~=2
                if lb(i)>=-gig1
                    if x(i)<lb(i)+eps1
                        x(i)=lb(i);
                        ire(i)=-1;
                        if mid==0
                            ip=-i;
                            [q,r,ip]=qr1(q,r,0,n,m,ip);
                            if ip==0
                                if imp==1
                                    fprintf('\n constraint %3d is added ',-i)
                                end
                                m=m+1;
                                ipvt(m)=-i;
                            end
                        end
                    end
                end
            end
            if ira>=2
                if(ub(i)<=gig1 && ire(i)==0)
                    if x(i)>(ub(i)-eps1)
                        x(i)=ub(i);
                        ire(i)=1;
                        if mid==0
                            ip=i;
                            [q,r,ip]=qr1(q,r,0,n,m,ip);
                            if ip==0
                                if imp==1
                                    fprintf('\n constraint %3d is added ',i)
                                end
                                m=m+1;%
                                ipvt(m)=i;
                            end
                        end
                    end
                end
            end
        end
    end
    for i=1:nin
        ii=neq+i;
        s=C(:,ii)'*x-b(ii);
        ni=n+i;
        if s>-eps1
            ire(ni)=1;
        else
            ire(ni)=0;
        end
        constraints(i,1)=s;
    end
    if mid>0
        if ira>0
            i=1;
            while i<=n && m<n
                ip=0;
                if ire(i)==1
                    ip=i;
                elseif ire(i)==-1
                    ip=-i;
                end
                if ip~=0
                    [q,r,ip]=qr1(q,r,0,n,m,ip);
                    if ip==0
                        if imp==1
                            fprintf('\n constraint %3d is added ',ire(i)*i)
                        end
                        m=m+1;
                        ipvt(m)=ire(i)*i;
                    else
                        ire(i)=0;
                    end
                end
                i=i+1;
            end
            if m==n
                for j=i:n+nin
                    ire(j)=0;
                end
            end
        end
        if nin>0
            i=1;
            ip=0;
            while i<=nin && m<n
                if ire(n+i)==1
                    [q,r,ip]=qr1(q,r,C(:,neq+i),n,m,ip);
                    if ip==0
                        if imp==1
                            fprintf('\n constraint %3d is added ',n+neq+i)
                        end
                        m=m+1;
                        ipvt(m)=n+i;
                    else
                        ire(n+i)=0;
                    end
                end
                i=i+1;
            end
            if m==n
                for j=i:nin
                    ire(j+n)=0;
                end
            end
        end
    end
end
iv=0;
if m==n
    minimo=1;
    id=2;
    nm=0;
else
    minimo=0;
end
if minimo==0 && modo~=-2
    m1=m+1;
    nm=n-m;
    [LT,jpvt,ind]=cholpar(H,n,q(:,m1:n),nm,iver-iv);
    if ind<=-n10
        ind=ind+n10;
    else
    end
end
if imp==1
    fprintf('\n Begining of iteractions')
end
while iter<=itermax
    if imp==1
        fprintf('\n Iteraction %3d',iter)
    end
    eig(q(:,m+1:n)'*H*q(:,m+1:n));
    ind1=0;
    inddes=0;
    il=0;
    if iicol==1
        id=2;
    end
    if icicla==0
        s=H*x;
        g(1:n,1)=s+p;
    end
    fx=0.5*x'*H*x+p'*x;%    
    if norm(q(:,m+1:n)'*g)/(1+abs(fx))<=eps1 && ind>=0%%
        id=2;
    end
    s1=gigant;
    s2=0;
    if id>=2
        for i=1:nm
            gred(i,1)=q(:,n1-i)'*g;
        end
        if minimo==0
            s2=norm(gred(1:nm))/(1+norm(g,'inf'));
        end
        if s2>=eps0
            info=10;
            w(1:nm)=gred(1:nm);
            icont=icont+1;
        elseif icont>0
            icont=0;
        end
        if m>mi1  && (icont==0 || icont==3)
            for i=1:m
                aux(i,1)=-q(1:n,i)'*g;
            end
            lambda=[];
            [lambda(1:m,1),ifallo]=solupptri(r(1:m,1:m),aux(1:m));
            if ifallo==1
                return
            end
            if m>mi1
                icol=0;
                j=nd-1+mi1;
                for i=mi1+1:m
                    j=j+1;
                    k=ipvt(i);
                    if k<=nmd
                        s=lambda(i);
                    end
                    if s<s1
                        s1=s;
                        icol=i;
                    end
                end
                if s1<=eps1
                    if icont>0
                        icont=0;
                    end
                    if m>1
                        [q,r]=qr2(q,r,n,m,icol);
                        if imp==1
                            fprintf('\n Constraint %3d removed',icol)
                        end
                    end
                    m1=m-1;
                    il=ipvt(icol);
                    if il>n
                        constraints(il-n)=0;
                    end
                    ire(abs(il))=0;
                    for j=icol:m1
                        ipvt(j)=ipvt(j+1);
                    end
                    ipvt(m)=0;
                    [LT,jpvt,ind]=cholpar(H,n,q(:,m1+1:n),n-m1,-1);
                    info=1;
                    m=m1;
                    nm=n-m;%
                end
            end
        end
    end
    if id>=2 && (s1>=-eps1 && inddes==0) && (s2<eps0 || icont>=3)
        fprintf('\n The process has been completed without problems')
        return
    end
    m1=m+1;
    if abs(inddes)<=n
        [d(1:n,1),inddes]=qpdes(q(1:n,m1:n),LT,g,g,w(1:2*n-1),jpvt,nm,n,ind,info,inddes);
        if imp==1
            if inddes==0
                if d'*(H)*d<-eps1
                    fprintf('\n A negative curvature direction has been calculated ');
                else
                    fprintf('\n A  null curvature direction has been calculated ');
                end
            else
                fprintf('\n A positive curvature direction has been calculated ');
            end
        end
    end
    id=inddes;
    k=0;
    if iver==-1 || iver~=iv
        if ira>0
            i=0;
            while i<n && k==0
                i1=i+1;
                xi1=x(i1);
                if ira>1
                    ubi=ub(i1);
                    if ubi<=gig1 && ire(i1)==0 && d(i1)>eps1 && xi1>=(ubi-eps1)
                        s2=norm(q(i1,m1:m1+nm-1));
                        if s2>=eps0
                            k=1;
                            ipvt(m1)=i1;
                        else
                            d(i1)=0;
                        end
                    end
                end
                if k==0 && ira ~=2
                    lbi=lb(i1);
                    if lbi>=-gig1 && ire(i1)==0 && d(i1)<-eps1 && xi1<=(lbi+eps1)
                        s2=norm(q(i1,m1:m1+nm-1));
                        if s2>=eps0
                            k=1;
                            ipvt(m1)=-i1;
                        else
                            d(i1)=0;
                        end
                    end
                end
                i=i1;
            end
        end
        i=0;
        while i<nin && k==0
            i=i+1;
            ni=n+i;
            if ire(ni)~=1
                cid(i)=C(1:n,neq+i)'*d;
                if constraints(i)>=-eps1 && d(i)>eps1
                    jj=1;
                    for j=m1:n
                        Zci(jj)=q(1:n,j)'*C(:,neq+i);
                        jj=jj+1;
                    end
                    s2=norm(Zci(1:nm));
                    if s2>=eps0
                        k=1;
                        ipvt(m1)=ni;
                    else
                        cid(i)=0;
                    end
                end
            end
        end
    end
    if k==0
        if id==0 && ind<=nm && ind>0
            id=2;
        end
        if ind1==11 && id~=1
            id=id+100;
        end
        if ind1==10 && id~=1
            id=id+10;
        end
        [ro,ipvt(m1),id]=qpstep(C,b,lb,ub,x,d,g,ire,ira,n,nin,neq,id);
        if imp==1
            if id==1 || id==3
                fprintf('\n step associated with the constraint %3d',ipvt(m1));
            end
        end
        if id==-1
            ind=-1;
            return
        elseif id==0
            id=2;
            k=1;
        end
        if id==1 || id==3 || id==11
            icol3=ipvt(m1);
        end
    end
    if k==1
        icicla=icicla+1;
        icol3=ipvt(m1);
        if (icicla >m && m>0) || (icol3==icol1 && icol==m)
            ind=-2;
            return
        end
        icol1=icol2;
        icol2=icol3;
    elseif icicla>0
        icicla=0;
        icol1=0;
        icol2=0;
    end
    if icicla==0
        for i=1:nin
            if ire(ni)==0 && (iver~=iv || iver==-1) && id<10
                if ro==1
                    constraints(i)=constraints(i)+cid(i);
                else
                    constraints(i)=constraints(i)+ro*cid(i);
                end
            end
        end
        if ro==1
            x=x+d;
        else
            x=x+ro*d;
        end
    end
    if icicla~=0 || id==1 || id==11 || id==3
        if il==-icol3
            iicol=1;
        else
            iicol=0;
        end
        s=LT;
        if icol3<0
            ire(-icol3)=-1;
        else
            ire(icol3)=1;
        end
        if icol3<=n
            ip=icol3;
            [q,r,ip]=qr1(q,r,0,n,m,ip);
            if ip==0
                if imp==1
                    fprintf('\n constraint %3d added ',icol3);
                end
                m=m1;
            else
                ire(abs(icol3))=0;
            end
        else
            ip=0;
            if icol3<=nmd
                [q,r,ip]=qr1(q,r,C(:,neq+icol3-n),n,m,ip);
                if ip==0
                    if imp==1
                        fprintf('\n constraint %3d added ',neq+icol3);
                    end
                    m=m1;
                else
                    ire(abs(icol3))=0;
                end
            end
        end
        nm=n-m;
        [LT,jpvt,ind]=cholpar(H,n,q(:,m+1:n),nm,-1);
        %         if nm>1
        %             if iibeta==1 && ind~=-2
        %                 ind=-1;
        %             end
        %             if ind>2*nm && in<3*nm
        %                 nf=3*nm-ind;
        %             elseif ind>nm && ind<2*nm
        %                 nf=2*nm-ind;
        %             elseif ind<-1 && ind>=-nm
        %                 nf=-ind-1;
        %             elseif ind==-1 || ind==-nm-1 || ind==3*nm || ind==2*nm
        %                 ind=0;
        %                 [LT,jpvt,ind]=cholpar(H,n,q(:,m+1:n),nm1,-1);
        %                 nf=0;
        %             elseif ind>=0
        %                 nf=nm
        %             else
        %                 nf=-nm-ind-1;
        %             end
        %         if nf~=0
        %             nmf=nm1-nf;
        %             if ind>nm
        %                 ii=nmf+i1;
        %             elseif ind<0
        %                 ii=nmf*(nmf+1)/2+i1;
        %             end
        %             if iibet==1
        %                 nf=nf-1;
        %             end
        %             [LT,jpvt,ind]=cholpar(H,n,q(:,m+1:n),nm,-1);
        %         end
        %         if ind<=-n10
        %             iibeta=1;
        %             ind=ind+n10;
        %         else
        %             iibeta=0;
        %         end
        %         if ind==n
        %             ind=-1;
        %             return
        %         end
        %         nm=nm1;
        %         end
    end
    
    if id==11
        fx=0.5*x'*H*x+p'*x;
        ind=-3;
        return
    end
    if m==n
        minimo=1;
        id=2;
        nm=0;
    else
        minimo=0;
    end
    fx=0.5*x'*H*x+p'*x;
    iter=iter+1;
end
ind=1;
fprintf('the number of iteractions has been exceeded');
end

